@@ -11,10 +11,13 @@ import distutils.log
 import distutils.core
 import distutils.cmd
 import distutils.dist
+import distutils.command
 from distutils.util import strtobool
 from distutils.debug import DEBUG
 from distutils.fancy_getopt import translate_longopt
 import itertools
+import textwrap
+from typing import List, Optional, TYPE_CHECKING
 
 from collections import defaultdict
 from email import message_from_file
@@ -29,11 +32,15 @@ from setuptools.extern import ordered_set
 from . import SetuptoolsDeprecationWarning
 
 import setuptools
+import setuptools.command
 from setuptools import windows_support
 from setuptools.monkey import get_unpatched
 from setuptools.config import parse_configuration
 import pkg_resources
 
+if TYPE_CHECKING:
+    from email.message import Message
+
 __import__('setuptools.extern.packaging.specifiers')
 __import__('setuptools.extern.packaging.version')
 
@@ -65,53 +72,75 @@ def get_metadata_version(self):
     return mv
 
 
+def rfc822_unescape(content: str) -> str:
+    """Reverse RFC-822 escaping by removing leading whitespaces from content."""
+    lines = content.splitlines()
+    if len(lines) == 1:
+        return lines[0].lstrip()
+    return '\n'.join(
+        (lines[0].lstrip(),
+         textwrap.dedent('\n'.join(lines[1:]))))
+
+
+def _read_field_from_msg(msg: "Message", field: str) -> Optional[str]:
+    """Read Message header field."""
+    value = msg[field]
+    if value == 'UNKNOWN':
+        return None
+    return value
+
+
+def _read_field_unescaped_from_msg(msg: "Message", field: str) -> Optional[str]:
+    """Read Message header field and apply rfc822_unescape."""
+    value = _read_field_from_msg(msg, field)
+    if value is None:
+        return value
+    return rfc822_unescape(value)
+
+
+def _read_list_from_msg(msg: "Message", field: str) -> Optional[List[str]]:
+    """Read Message header field and return all results as list."""
+    values = msg.get_all(field, None)
+    if values == []:
+        return None
+    return values
+
+
 def read_pkg_file(self, file):
     """Reads the metadata values from a file object."""
     msg = message_from_file(file)
 
-    def _read_field(name):
-        value = msg[name]
-        if value == 'UNKNOWN':
-            return None
-        return value
-
-    def _read_list(name):
-        values = msg.get_all(name, None)
-        if values == []:
-            return None
-        return values
-
     self.metadata_version = StrictVersion(msg['metadata-version'])
-    self.name = _read_field('name')
-    self.version = _read_field('version')
-    self.description = _read_field('summary')
+    self.name = _read_field_from_msg(msg, 'name')
+    self.version = _read_field_from_msg(msg, 'version')
+    self.description = _read_field_from_msg(msg, 'summary')
     # we are filling author only.
-    self.author = _read_field('author')
+    self.author = _read_field_from_msg(msg, 'author')
     self.maintainer = None
-    self.author_email = _read_field('author-email')
+    self.author_email = _read_field_from_msg(msg, 'author-email')
     self.maintainer_email = None
-    self.url = _read_field('home-page')
-    self.license = _read_field('license')
+    self.url = _read_field_from_msg(msg, 'home-page')
+    self.license = _read_field_from_msg(msg, 'license')
 
     if 'download-url' in msg:
-        self.download_url = _read_field('download-url')
+        self.download_url = _read_field_from_msg(msg, 'download-url')
     else:
         self.download_url = None
 
-    self.long_description = _read_field('description')
-    self.description = _read_field('summary')
+    self.long_description = _read_field_unescaped_from_msg(msg, 'description')
+    self.description = _read_field_from_msg(msg, 'summary')
 
     if 'keywords' in msg:
-        self.keywords = _read_field('keywords').split(',')
+        self.keywords = _read_field_from_msg(msg, 'keywords').split(',')
 
-    self.platforms = _read_list('platform')
-    self.classifiers = _read_list('classifier')
+    self.platforms = _read_list_from_msg(msg, 'platform')
+    self.classifiers = _read_list_from_msg(msg, 'classifier')
 
     # PEP 314 - these fields only exist in 1.1
     if self.metadata_version == StrictVersion('1.1'):
-        self.requires = _read_list('requires')
-        self.provides = _read_list('provides')
-        self.obsoletes = _read_list('obsoletes')
+        self.requires = _read_list_from_msg(msg, 'requires')
+        self.provides = _read_list_from_msg(msg, 'provides')
+        self.obsoletes = _read_list_from_msg(msg, 'obsoletes')
     else:
         self.requires = None
         self.provides = None
@@ -598,7 +627,8 @@ class Distribution(_Distribution):
                         continue
 
                     val = parser.get(section, opt)
-                    opt = self.dash_to_underscore_warning(opt, section)
+                    opt = self.warn_dash_deprecation(opt, section)
+                    opt = self.make_option_lowercase(opt, section)
                     opt_dict[opt] = (filename, val)
 
             # Make the ConfigParser forget everything (so we retain
@@ -623,12 +653,18 @@ class Distribution(_Distribution):
             except ValueError as e:
                 raise DistutilsOptionError(e) from e
 
-    def dash_to_underscore_warning(self, opt, section):
+    def warn_dash_deprecation(self, opt, section):
         if section in (
             'options.extras_require', 'options.data_files',
         ):
             return opt
+
         underscore_opt = opt.replace('-', '_')
+        commands = distutils.command.__all__ + setuptools.command.__all__
+        if (not section.startswith('options') and section != 'metadata'
+                and section not in commands):
+            return underscore_opt
+
         if '-' in opt:
             warnings.warn(
                 "Usage of dash-separated '%s' will not be supported in future "
@@ -636,6 +672,18 @@ class Distribution(_Distribution):
                 % (opt, underscore_opt))
         return underscore_opt
 
+    def make_option_lowercase(self, opt, section):
+        if section != 'metadata' or opt.islower():
+            return opt
+
+        lowercase_opt = opt.lower()
+        warnings.warn(
+            "Usage of uppercase key '%s' in '%s' will be deprecated in future "
+            "versions. Please use lowercase '%s' instead"
+            % (opt, section, lowercase_opt)
+        )
+        return lowercase_opt
+
     # FIXME: 'Distribution._set_command_options' is too complex (14)
     def _set_command_options(self, command_obj, option_dict=None):  # noqa: C901
         """
