@@ -10,6 +10,8 @@ from setuptools.dist import (
     check_package_data,
     DistDeprecationWarning,
     check_specifier,
+    rfc822_escape,
+    rfc822_unescape,
 )
 from setuptools import sic
 from setuptools import Distribution
@@ -85,6 +87,9 @@ def __read_test_cases():
         ('Metadata version 1.1: Provides', params(
             provides=['package'],
         )),
+        ('Metadata Version 1.0: Short long description', params(
+            long_description='Short long description',
+        )),
         ('Metadata version 1.1: Obsoletes', params(
             obsoletes=['foo'],
         )),
@@ -162,6 +167,7 @@ def test_read_metadata(name, attrs):
         ('metadata_version', dist_class.get_metadata_version),
         ('provides', dist_class.get_provides),
         ('description', dist_class.get_description),
+        ('long_description', dist_class.get_long_description),
         ('download_url', dist_class.get_download_url),
         ('keywords', dist_class.get_keywords),
         ('platforms', dist_class.get_platforms),
@@ -336,3 +342,37 @@ def test_check_specifier():
     attrs = {'name': 'foo', 'python_requires': ['>=3.0', '!=3.1']}
     with pytest.raises(DistutilsSetupError):
         dist = Distribution(attrs)
+
+
+@pytest.mark.parametrize(
+    'content, result',
+    (
+        pytest.param(
+            "Just a single line",
+            None,
+            id="single_line",
+        ),
+        pytest.param(
+            "Multiline\nText\nwithout\nextra indents\n",
+            None,
+            id="multiline",
+        ),
+        pytest.param(
+            "Multiline\n    With\n\nadditional\n  indentation",
+            None,
+            id="multiline_with_indentation",
+        ),
+        pytest.param(
+            "  Leading whitespace",
+            "Leading whitespace",
+            id="remove_leading_whitespace",
+        ),
+        pytest.param(
+            "  Leading whitespace\nIn\n    Multiline comment",
+            "Leading whitespace\nIn\n    Multiline comment",
+            id="remove_leading_whitespace_multiline",
+        ),
+    )
+)
+def test_rfc822_unescape(content, result):
+    assert (result or content) == rfc822_unescape(rfc822_escape(content))
